@@ -66,6 +66,34 @@ class TestDatabaseAPI(base.TestCase):
         res = api.get_test_by_test_id('fake_test')
         self.assertIsNone(res)
 
+    def test_get_tests_by_test_ids(self):
+        test_a = api.create_test('fake_test1', 2, 1, 1, 1.2)
+        test_b = api.create_test('fake_test2', 4, 2, 2, 2.3)
+        test_c = api.create_test('fake_test3', 6, 3, 3, 3.3)
+        test_d = api.create_test('fake_test4', 8, 4, 4, 4.3)
+        result = api.get_tests_by_test_ids(
+            ['fake_test1', 'fake_test2', 'fake_test3'])
+        result_ids = [x.id for x in result]
+        self.assertIn(test_a.id, result_ids)
+        self.assertIn(test_b.id, result_ids)
+        self.assertIn(test_c.id, result_ids)
+        self.assertNotIn(test_d.id, result_ids)
+
+    def test_get_tests_by_test_ids_no_matches(self):
+        api.create_test('fake_test5', 2, 1, 1, 1.2)
+        api.create_test('fake_test6', 4, 2, 2, 2.3)
+        api.create_test('fake_test7', 6, 3, 3, 3.3)
+        api.create_test('fake_test8', 8, 4, 4, 4.3)
+        result = api.get_tests_by_test_ids(
+            ['fake_test1', 'fake_test2', 'fake_test3'])
+        self.assertEqual([], result)
+
+    def test_get_test_with_sql_injection(self):
+        api.create_test("test_terror'); DROP TABLE tests;")
+        res = api.get_all_tests()
+        self.assertEqual(len(res), 1)
+        self.assertEqual(res[0].test_id, "test_terror'); DROP TABLE tests;")
+
     def test_create_run_and_list(self):
         res = api.create_run()
         self.assertIsNotNone(res)
@@ -226,6 +254,45 @@ class TestDatabaseAPI(base.TestCase):
         id_value = api.get_id_from_test_id('fake_test')
         self.assertEqual(test_a.id, id_value)
 
+    def test_get_run_times_all_test_runs(self):
+        timestamp_a = datetime.datetime.utcnow()
+        timestamp_b = timestamp_a + datetime.timedelta(seconds=3)
+        run_a = api.create_run()
+        run_b = api.create_run()
+        test_a = api.create_test('test_a')
+        test_b = api.create_test('test_b')
+        api.create_test_run(test_a.id, run_a.id, 'success', timestamp_a,
+                            timestamp_b)
+        api.create_test_run(test_a.id, run_b.id, 'success', timestamp_a,
+                            timestamp_b)
+        api.create_test_run(test_b.id, run_b.id, 'success', timestamp_a,
+                            timestamp_b)
+        res = api.get_run_times_all_test_runs()
+        expected_dict = {
+            'test_a': [3, 3],
+            'test_b': [3]
+        }
+        self.assertEqual(expected_dict, res)
+
+    def test_get_run_times_all_test_runs_with_tests_filter(self):
+        timestamp_a = datetime.datetime.utcnow()
+        timestamp_b = timestamp_a + datetime.timedelta(seconds=3)
+        run_a = api.create_run()
+        run_b = api.create_run()
+        test_a = api.create_test('test_a')
+        test_b = api.create_test('test_b')
+        api.create_test_run(test_a.id, run_a.id, 'success', timestamp_a,
+                            timestamp_b)
+        api.create_test_run(test_a.id, run_b.id, 'success', timestamp_a,
+                            timestamp_b)
+        api.create_test_run(test_b.id, run_b.id, 'success', timestamp_a,
+                            timestamp_b)
+        res = api.get_run_times_all_test_runs(tests=['test_a'])
+        expected_dict = {
+            'test_a': [3, 3],
+        }
+        self.assertEqual(expected_dict, res)
+
     def test_get_test_runs_by_run_id(self):
         run_b = api.create_run()
         run_a = api.create_run()
@@ -420,7 +487,7 @@ class TestDatabaseAPI(base.TestCase):
         self.assertIn(timestamp_b, result.keys())
         for timestamp in result:
             if timestamp.replace(
-                microsecond=0) == timestamp_a.replace(microsecond=0):
+                    microsecond=0) == timestamp_a.replace(microsecond=0):
                 self.assertEqual(5, result[timestamp])
             else:
                 self.assertEqual(2, result[timestamp])
@@ -546,6 +613,49 @@ class TestDatabaseAPI(base.TestCase):
         self.assertEqual(1, len(api.get_test_run_metadata(test_run_b.id)))
         self.assertEqual(0, len(api.get_test_run_metadata(test_run_a.id)))
 
+    def test_delete_run_by_uuid(self):
+        metadata_a = {'key': 'This is the metadata we\'re keeping'}
+        metadata_b = {'key': 'This metadata should be removed'}
+        run_a = api.create_run()
+        run_b = api.create_run()
+        uuid_to_delete = run_a.uuid
+        api.add_run_metadata(metadata_a, run_a.id)
+        api.add_run_metadata(metadata_b, run_b.id)
+        api.delete_run_by_uuid(uuid_to_delete)
+        runs = api.get_all_runs()
+        self.assertEqual(1, len(runs))
+        self.assertEqual(1, api.get_session().query(
+            models.RunMetadata.id).count())
+        self.assertEqual(run_b.id, runs[0].id)
+        self.assertEqual(1, len(api.get_run_metadata(run_b.uuid)))
+        self.assertEqual(0, len(api.get_run_metadata(run_a.uuid)))
+
+    def test_delete_test_runs_by_run_uuid(self):
+        # create runs
+        run_a = api.create_run()
+        run_b = api.create_run()
+        uuid_to_delete = run_a.uuid
+        # create test runs
+        test = api.create_test('fake_test')
+        test_run_a = api.create_test_run(test.id, run_a.id, 'fail',
+                                         start_time=datetime.datetime(
+                                             1914, 6, 28, 10, 45, 0))
+        test_run_b = api.create_test_run(test.id, run_b.id, 'fail',
+                                         start_time=datetime.datetime.utcnow())
+        # initialize and metadata to test_runs
+        metadata_a = {
+            'key': 'this test belongs to the run we are deleting from'}
+        metadata_b = {
+            'key': 'this test belongs to the run we are leaving alone'}
+        api.add_test_run_metadata(metadata_a, test_run_a.id)
+        api.add_test_run_metadata(metadata_b, test_run_b.id)
+        api.delete_test_runs_by_run_uuid(uuid_to_delete)
+        test_runs = api.get_all_test_runs()
+        self.assertEqual(1, len(test_runs))
+        self.assertEqual(test_run_b.id, test_runs[0].id)
+        self.assertEqual(1, len(api.get_test_run_metadata(test_run_b.id)))
+        self.assertEqual(0, len(api.get_test_run_metadata(test_run_a.id)))
+
     def test_get_run_metadata(self):
         run_a = api.create_run()
         run_b = api.create_run()
@@ -909,6 +1019,15 @@ class TestDatabaseAPI(base.TestCase):
         self.assertEqual(1, len(results))
         self.assertEqual(run_c.id, results[0].id)
 
+    def test_get_runs_count(self):
+        for i in range(0, 100):
+            api.create_run()
+        count = api.get_runs_count()
+        self.assertEqual(100, count)
+
+    def test_get_runs_count_zero(self):
+        self.assertEqual(0, api.get_runs_count())
+
     def test_get_runs_counts_by_run_metadata_no_start_date(self):
         run_a = api.create_run(fails=1)
         api.create_run()
